/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: opendir.c,v 1.9 1994/11/26 14:51:53 mura Exp $
 */

/* System headers */
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/dos.h>
#include <sys/xstat.h>
#include <sys/xstdlib.h>
#include <sys/xglob.h>
#include <sys/xtime.h>
#include <sys/xunistd.h>

/* Macros */
#define SEARCH_ATTRIB	 (0xff)
#define ROOTDIRECTORY(p) (p[0] && p[1] == ':' && p[2] == '/' && p[3] == '\0')
#define DOTDOT(p)	 (p[0] == '.' && p[1] == '.' && p[2] == '\0')
#define DOT(p)		 (p[0] == '.' && p[1] == '\0')
#define MAXDIR		 (128)
#define DIRSIZE(x)	 (sizeof (struct dirent) * MAXDIR * (((x) == 0) ? 1 : (x) >> 6))
#define PASSCURDRV(x)	 ((x) + 1)

/* File scope variables */
static short int _days_in_month[12] = {
    0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
};

/* File scope variables */
static const char exec_suffix[][2] = {
    { "x" }, { "r" }
};
#define EXEC_SUFFIX_NUM (sizeof (exec_suffix) / sizeof (exec_suffix[0]))

/* File scope functions */
static int _executable (const char *name)
{
    int i;
    char *ext;

    /* sIḧʒuT */
    if ((ext = strrchr (name, '.')) == 0)
	return 0;

    /* |C^̕␳ */
    ext++;

    /* gqXg猟 */
    for (i = 0; i < EXEC_SUFFIX_NUM; i++)
	if (stricmp (ext, exec_suffix[i]) == 0)
	    return 1;

    /* I */
    return 0;
}

/* Functions */
DIR *opendir (const char *name)
{
    int days;
    int cdromflg;
    DIR *dirp;
    struct dirent *dp;
    char nbuf[PATH_MAX + 1];
    char exppath[PATH_MAX + 1];
    union {
	int dostime;
	tstamp_t tm;
    } ts;

    /* tpXɓWJăGg[ׂ͒ */
    _fullpath (exppath, name, sizeof exppath);

    /* CDROM hCuǂ肷 */
    cdromflg = _iscdromdrive (PASSCURDRV (exppath[0] - 'A'));

    /* [gfBNg̃XbVO */
    if (ROOTDIRECTORY (exppath))
	exppath[2] = '\0';

    /* pXɃChJ[h */
    strcat (strcpy (nbuf, exppath), "/*.*");

    /* *dirp  DIR ̗̈mۂ */
    if ((dirp = (DIR *) malloc (sizeof (DIR))) == 0)
	goto error;

    /*  */
    dirp->loc = dirp->filemax = 0;

    /* *dirp->dp  dirent ̗̈mۂ */
    if ((dirp->dp = (struct dirent *) malloc (DIRSIZE (0))) == 0)
	goto free_2;

    /* Ƃ肠 FILES Ċm߂ */
    if (_dos_files (&dirp->filesbuf, nbuf, SEARCH_ATTRIB) < 0)
	goto free_1;

    /* ǂݍ */
    do {

	/* dirent[dirp->filemax] \ */
	dp = &dirp->dp[dirp->filemax];

        /* i-node ߂ (CDROM ̓_~[ݒ) */
	if (cdromflg)
	    dp->d_ino = 0;
	else if ((dp->d_ino = _getinode (&dirp->filesbuf, WILD_TRUE)) < 0)
	    goto free_1;

	/* t@CƁA̒߂ */
	dp->d_namlen = dp->d_reclen =
	    strlen (strcpy (dp->d_name, dirp->filesbuf.name));

	/* t@C[hZo */
	dp->d_mode = _mode2unix (dirp->filesbuf.atr);

	/* t@C[h̏C */
	if (_DOS_ISVOL (dirp->filesbuf.atr))
	    dp->d_mode &= ~(S_IREAD | S_IWRITE);

	/* CDROM  S_IWRITE 폜 */
	if (cdromflg)
	    dp->d_mode &= ~S_IWRITE;

	if (_executable (dirp->filesbuf.name))
	    dp->d_mode |= S_IEXEC;

	/* ^CX^v̎擾 */
	ts.dostime = (dirp->filesbuf.date << 16) | dirp->filesbuf.time;

	/* g[^̌vZ */
	days = ((ts.tm.year + 10) * 365
		+ (_days_in_month[ts.tm.month - 1])
		+ (ts.tm.day)
		+ ((ts.tm.year >> 2) + 2)
		- (((ts.tm.year & 3) == 0 && ts.tm.month < 3) ? 1 : 0));

	/* Ԃ̐ݒ/g[^b̌vZ (ŏ JST Ȃ̂ŕ␳) */
	dp->d_time = (((days * 24 + ts.tm.hour) * 60
		       + ts.tm.minute) * 60
		      + (ts.tm.sec2 << 1) + _TZADJUST);

	/* t@CTCYݒ */
	dp->d_size = dirp->filesbuf.filelen;

	/* t@CJEg */
	dirp->filemax++;

	/* dirent ̈悪ȂΊg */
	if ((dirp->filemax % MAXDIR) == 0)
	    if ((dirp->dp = realloc (dirp->dp, DIRSIZE (dirp->filemax))) == 0)
		return 0;

	/* t@CȂȂ܂ő */

    }
#ifdef __LIBSXC__
    while (TSNFiles (&dirp->filesbuf, nbuf, SEARCH_ATTRIB) == 0);
#else
    while (_dos_nfiles (&dirp->filesbuf) == 0);
#endif

    /* dirp Ԃ */
    return dirp;

  free_1:
    /*  */
    free (dirp->dp);

  free_2:
    /*  */
    free (dirp);

  error:
    /* G[ŏI */
    errno = ENOENT;
    return 0;
}
