/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * based on the contribution from TomY.
 * --------------------------------------------------------------------
 * $Id: ftw.c,v 1.1 1993/06/06 06:06:40 mura Exp $
 */

/* System headers */
#include <stdlib.h>
#include <dirent.h>
#include <errno.h>
#include <ftw.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/xglob.h>

/* File scope functions */
static int _ftwsub (const char *path,
		    int (*fn) (const char *, const struct stat *, int),
		    int depmax, int depth)
{
    int r, inf;
    struct dirent *entp;
    struct stat st;
    char ename[PATH_MAX + 1];
    DIR *dirp;

    /* w肵fBNgI[v */
    if ((dirp = opendir (path)) == 0)
	return -1;

    /* ܂܂IuWFNg𒲂ׂ */
    while ((entp = readdir (dirp)) != NULL) {

	/* IuWFNg̃tpX𓾂 */
	strcpy (ename, path);
	_addlastsep (ename);
	strcat (ename, entp->d_name);

	/* lstat ɂđΏۂ𒲍 */
	if (lstat (ename, &st) < 0)
	    inf = FTW_NS;

	/* V{bNNȂ... */
	else if (S_ISLNK (st.st_mode))
	    inf = FTW_SL;

	/* {[t@CȂ... */
	else if (st.st_mode & S_IFVOL)
	    inf = FTW_VL;

	/* fBNgȂ... */
	else if (S_ISDIR (st.st_mode))
	    inf = FTW_D;

	/* ʂ̃t@CȂ... */
	else
	    inf = FTW_F;

	/* ֐Ăяo */
	if ((r = fn (ename, &st, inf)) != 0) {

	    /* fBNg */
	    closedir (dirp);

	    /* r Ԃ */
	    return r;

	}

	/* hbgt@CȂA[]T΃fBNgǂ */
	if (inf == FTW_D && depth < depmax &&
	    strcmp (entp->d_name, ".") != 0 && strcmp (entp->d_name, "..") != 0) {

		/* ̃fBNgɃgo[X */
		r = _ftwsub (ename, fn, depmax, depth + 1);

	}

    }

    /* fBNg */
    closedir (dirp);

    /* 0 Ԃ */
    return 0;
}

/* Functions */
int ftw (const char *path,
	 int (*fn) (const char *, const struct stat *, int), int ndirs)
{
    char ename[PATH_MAX + 1];

    /* ndir `FbN */
    if (ndirs <= 0) {
	errno = EINVAL;
	return -1;
    }

    /* tpXɓWJ */
    _fullpath (ename, path, sizeof ename);

    /* Tu[`Ăяo */
    return _ftwsub (ename, fn, ndirs, 1);
}
