/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id:$
 */

/* System headers */
#include <math.h>
#include <errno.h>
#include <sys/xmath.h>

/* Functions */
double atan2 (double y, double x)
{
    double pi;

    if (x > 0.0) {
	asm volatile ("fatan.x %1,%0" : "=f" (x) : "f" (y / x));
	FPU_ERRCHK ();
	return x;
    }

    asm volatile ("fmovecr.x #$0,%0" : "=f" (pi));

    if (x < 0.0) {
	if (y > 0.0) {
	    asm volatile ("fatan.x %1,%0" : "=f" (x):"f" (y / x));
	    FPU_ERRCHK ();
	    return x + pi;
	}
	else if (y < 0.0) {
	    asm volatile ("fatan.x %1,%0" : "=f" (x):"f" (y / x));
	    FPU_ERRCHK ();
	    return x - pi;
	}
	else
	    return pi;
    }
    else {
	double pi_2;

	asm volatile ("fscale.w #-1,%0" : "=f" (pi_2) : "0" (pi));

	if (y < 0.0)
	    return -pi_2;
	else if (y > 0.0)
	    return +pi_2;
	else {
	    errno = EDOM;
	    return _NAN;
	}
    }
}
