
/* System headers */
#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/xstart.h>

/* External varibles */
const char __progname_dummy[] = "";

/* File scope functions */
static const char *progname (void);

/* Macros */
#define VA(call)		\
    {				\
	va_list ap;		\
	va_start (ap, fmt);	\
	call;			\
	va_end (ap);		\
    }

/* Functions */
void __volatile err (int stat, const char *fmt, ...)
{
    VA (verr (stat, fmt, ap));
}

/* Functions */
void __volatile verr (int stat, const char *fmt, va_list ap)
{
    vwarn (fmt, ap);
    exit (stat);
}

/* Functions */
void __volatile errx (int stat, const char *fmt, ...)
{
    VA (verrx (stat, fmt, ap));
}

/* Functions */
void __volatile verrx (int stat, const char *fmt, va_list ap)
{
    vwarnx (fmt, ap);
    exit (stat);
}

/* Functions */
void warn (const char *fmt, ...)
{
    VA (vwarn (fmt, ap));
}

/* Functions */
void vwarn (const char *fmt, va_list ap)
{
    int err = errno;
    const char* pname = progname ();

    if (pname)
	eprintf ("%s: ", pname);
    if (fmt) {
	vfprintf (stderr, fmt, ap);
	eprintf (": ");
    }
    eprintf ("%s\n", strerror (err));
}

/* Functions */
void warnx (const char *fmt, ...)
{
    VA (vwarnx (fmt, ap));
}

/* Functions */
void vwarnx (const char *fmt, va_list ap)
{
    const char* pname = progname ();

    if (pname)
	eprintf ("%s: ", pname);
    if (fmt)
	vfprintf (stderr, fmt, ap);
    eprintf ("\n");
}

/* File scope functions */
static const char *progname (void)
{
    const char *p = __progname;

    if (p == __progname_dummy) {
	const char *name = _argv[0];

	if (name[0] && name[1] == ':')
	    name += 2;
	if ((p = strrchr (name, '/')) != NULL)
	    p++;
	else
	    p = name;
	__progname = (char *) p;
    }
    return p;
}

/* EOF */
