/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: _sigdfl.c,v 1.4 1993/06/22 07:13:45 mura Exp $
 */

/* System headers */
#include <a_out.h>
#include <stdio.h>
#include <signal.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/dos.h>
#include <sys/iocs.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/xstart.h>
#include <sys/xsignal.h>
#include <sys/xresource.h>

/* External variables */
volatile struct sigaction _sigactions[SIG_MAX] = {
    { SIG_DFL, 0, SA_FORCEEXIT },		/* 0 */
    { SIG_DFL, 0, SA_DUMPCORE  },		/* SIGABRT */
    { SIG_DFL, 0, SA_DUMPCORE  },		/* SIGFPE  */
    { SIG_DFL, 0, SA_DUMPCORE  },		/* SIGILL  */
    { SIG_DFL, 0, SA_FORCEEXIT },		/* SIGINT  */
    { SIG_DFL, 0, SA_DUMPCORE  },		/* SIGSEGV */
    { SIG_DFL, 0, SA_FORCEEXIT },		/* SIGTERM */
    { SIG_DFL, 0, SA_FORCEEXIT },		/* SIGALRM */
    { SIG_DFL, 0, SA_DUMPCORE  },		/* SIGKILL */
    { SIG_DFL, 0, SA_DUMPCORE  },		/* SIGBUS  */
    { SIG_DFL, 0, SA_STOPPROC  },		/* SIGSTOP */
    { SIG_DFL, 0, SA_DUMPCORE  },		/* SIGEMT  */
    { SIG_DFL, 0, SA_FORCEEXIT },		/* SIGPIPE */
    { SIG_DFL, 0, SA_FORCEEXIT },		/* SIGHUP  */
    { SIG_DFL, 0, SA_DUMPCORE  },		/* SIGQUIT */
    { SIG_DFL, 0, SA_FORCEEXIT },		/* SIGUSR1 */
    { SIG_DFL, 0, SA_FORCEEXIT },		/* SIGUSR2 */
};

/* File scope functions */
static void _print (const char *format, ...)
{
    va_list ap;
    char buf[256];

    /* tH[}bgo */
    va_start (ap, format);
    vsnprintf (buf, sizeof buf, format, ap);
    va_end (ap);
    _dos_c_print (buf);
}

/* File scope functions */
static void _sigcore (einfo_t *info, int size)
{
    int fd;
    int bytes;
    struct corehdr co;

    /* dump oCg߂ (size = -1  Unlimited) */
    bytes = _memcp->block_end - (void *) _memcp;
    if (size > 0 && bytes > size)
	bytes = size;

    /* SĂ̗̈NA */
    memset (&co, '\0', sizeof (co));

    /* t@CL^ */
    strcpy (co.fname, _argv[0]);

    /* wb_쐬 */
    co.magic = CMAGIC;
    co.pc    = info->pc;
    co.sp    = info->sp;
    co.sr    = info->sr;
    co.memcp = (long) _memcp;
    co.procp = (long) _procp;
    co.psta  = (long) _psta;
    co.dsta  = (long) _dsta;
    co.bsta  = (long) _bsta;
    co.csta  = (long) _csta;
    co.vsta  = (long) _vsta;
    co.esta  = (long) _esta;
    co.fsta  = (long) _fsta;
    co.ssta  = (long) _ssta;
    co.hsta  = (long) _hsta;
    co.last  = (long) _last;
    co.mmax  = (long) _mmax;

    /* RAt@CI[v */
    if ((fd = _dos_create ("core", _DOS_IFREG)) < 0)
	return;

    /* RAt@C̃wb_ */
    if (_dos_write (fd, (char *) &co, sizeof co) < 0)
	return;

    /* C[W_v */
    if (_dos_write (fd, (char *) _memcp, bytes) < 0)
	return;

    /* t@CN[Y */
    _dos_close (fd);
}

/* Functions */
void _sigdfl (int sig, einfo_t *info)
{
    int mode, limit;
    union wait r;

    /* 샂[h𒲂ׂ */
    mode  = _sigactions[sig].sa_flags & SA_MODEMASK;
    limit = _limits[RLIMIT_CORE].rlim_cur;

    /* RA_vł邩ǂ`FbN */
    if (mode == SA_DUMPCORE && limit == 0)
	mode = SA_FORCEEXIT;

    /* IR[hNA */
    r.w_status = 0;
    r.w_termsig = sig;

    /* ftHg̓𒲂ׂ */
    switch (mode) {

    /* SA_FORCEEXIT ͋I */
    case SA_FORCEEXIT:
	_print ("%s\r\n", strsignal (sig));
	break;

    /* SA_DUMPCORE ̓RA_v */
    case SA_DUMPCORE:
	_sigcore (info, limit);
	_print ("%s (core dumped)\r\n", strsignal (sig));
	r.w_coredump = 1;
	break;

    /* SA_SIGIGN  SA_STOPPROC ͉Ȃ */
    default:
	return;
    }

    /* o͂ */
    _print ("pc:%#x, sp:%#x, sr:0x%04x\r\n", info->pc, info->sp, info->sr);

    /* vO exit  */
    _exit (r.w_status);
}
