/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: _path.c,v 1.4 1994/07/27 13:50:00 mura Exp $
 */

/* System headers */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/dos.h>
#include <sys/xglob.h>
#include <sys/xstdlib.h>

/* Macros */
#define ROOTDIRECTORY(p) (memcmp (((p) + 1), ":/", 3) == 0)
#define DOTDOT(p)	 (p[0] == '.' && p[1] == '.' && p[2] == '\0')
#define DOT(p)		 (p[0] == '.' && p[1] == '\0')
#define NOMAL		 (0x40)
#define VIRDRV		 (0x50)
#define VIRDIR		 (0x60)

/* File scope functions */
static void entrypress (char *path, int mode)
{
    char *p, *e, *t;
    char cat[PATH_MAX + 1];
    char tmp[PATH_MAX + 1];

    /* Rs[ */
    strcpy (tmp, path);

    /* Ō / ̈ʒuL */
    e = strrchr (tmp, '/') + 1;

    /* [gfBNgǂ */
    if (ROOTDIRECTORY (path) || (p = strtok2 (tmp, "/")) == 0) {
	strupr (path);
	return;
    }

    /* pXo */
    strcpy (path, p);
    strupr (path);

    /* pX\vfo */
    while ((p = strtok2 (0, "/")) != NULL) {

	/* .. Oɖ߂ */
	if (DOTDOT (p)) {
	    if (mode && p == e) {
		cat[0] = '/';
		strcpy (cat + 1, p);
		strcat (path, cat);
	    }
	    else if ((t = strrchr (path, '/')) > tmp + 2)
		*t = '\0';
	}

	/* . 炻ׂ̕ */
	else if (!DOT (p) || (mode && p == e)) {
	    cat[0] = '/';
	    strcpy (cat + 1, p);
	    strcat (path, cat);
	}

    }

    /* Ō̒ */
    if (path[2] == '\0')
	strcat (path, "/");
}

/* File scope functions */
static void _physpath (char *dst, const char *src, int mode)
{
    int drvno;
    char cat[PATH_MAX + 1];
    char tmp[PATH_MAX + 1];

    /* hCuԍ߁Atmp ɒ` */
    drvno = _getdriveno (src);
    tmp[0] = drvno + 'A';
    tmp[1] = ':';
    tmp[2] = '/';
    tmp[3] = '\0';

    /* whCũJgpX߁Atmp ɘA */
    _dos_curdir (drvno + 1, tmp + 3);

    /* [gfBNgȂ */
    if (*src == '/' || *src == '\\')
	strcpy (tmp + 3, src);

    /* hCu... */
    else if (src[1] == ':') {

	/* hCut[gfBNg */
	if (src[2] == '/' || src[2] == '\\')
	    strcpy (tmp, src);

	/* hCutt@C */
	else {
	    cat[0] = '/';
	    strcpy (cat + 1, src + 2);
	    strcat (tmp, cat);
	}

    }

    /* t@Ĉ */
    else {
	cat[0] = '/';
	strcpy (cat + 1, src);
	strcat (tmp, cat);
    }

    /* pXXbVɂ */
    _toslash (tmp);

    /* fBNgGgׂ */
    entrypress (tmp, mode);

    /* Rs[ďI */
    strcpy (dst, tmp);
}

/* File scope functions */
static __inline void ptrswap (char **p1, char **p2)
{
    char *tmp;

    /*  */
    tmp = *p2;
    *p2 = *p1;
    *p1 = tmp;
}

/* Functions */
char *_pathexpand (char *dst, const char *src, size_t maxlen, int mode)
{
    int normalpath;
    char drive[4];
    char tmpbuf1[PATH_MAX + 1];
    char tmpbuf2[PATH_MAX + 1];
    char virpath[PATH_MAX + 1];
    char *srcptr, *dstptr;

    /* Rs[ */
    srcptr = strcpy (tmpbuf1, src);
    dstptr = tmpbuf2;

    /* zhCu/fBNgm[}pXɂȂ܂œWJ */
    for (normalpath = 0; normalpath == 0;) {
	int status, drv;

	/* hCuԍo */
	if ((drv = _getdriveno (srcptr)) < 0) {
	    errno = _errcnv (drv);
	    return 0;
	}

	/* hCu߂ */
	drive[0] = drv + 'A';
	drive[1] = ':';
	drive[2] = '\0';

	/* ATCo */
	if ((status = _dos_getassign (drive, dstptr)) < 0) {
	    errno = _errcnv (status);
	    return 0;
	}

	/* ATC */
	switch (status) {

	case NOMAL:				/* ʏ̃t@C */
	    _physpath (dstptr, srcptr, mode);
	    normalpath = 1;
	    break;

	case VIRDRV:				/* zhCu */
	    if (drv == _getdriveno (dstptr)) {

		/* hCuȂȂ */
		_physpath (dstptr, srcptr, mode);
		normalpath = 1;
		break;
	    }
	    /* ႤhCuȂ牼zfBNgƓ */

	case VIRDIR:				/* zfBNg */
	    _physpath (virpath, srcptr, mode);
	    strcat (dstptr, virpath + 2);
	    ptrswap (&dstptr, &srcptr);
	    break;

	default:				/* G[ */
	    errno = EFAULT;
	    return 0;
	}

    }

    /* Ō̃XbV菜 */
    _dellastsep (dstptr);

    /* obt@ɓ肫邩 */
    if (strlen (dstptr) >= maxlen) {
	errno = ERANGE;
	return 0;
    }

    /* Rs[ dst Ԃ */
    return strcpy (dst, dstptr);

}
