/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: system.c,v 1.3 1994/05/02 16:53:08 mura Exp $
 */

/* System headers */
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/dos.h>
#include <sys/iocs.h>
#include <sys/xglob.h>
#include <sys/xtime.h>
#include <sys/xsignal.h>

/* Macros */
#define STDFDS 5
#define SYSTEM_ERR (127 << 8)

/* Functions */
int system (const char *cmdline)
{
    int i, len;
    int rc, interactive;
    char *shell = 0;
    char *shellopt = 0;
    char *shelltype = 0;
    enum {
	SHTYPE_COMMAND,
	SHTYPE_UNIX,
	SHTYPE_UNKNOWN
    } shtype;

    /* cmdline  0 ȂΑΘbIVFN */
    interactive = (cmdline == 0 || *cmdline == '\0');

    /* ΘbIłȂꍇ... */
    if (!interactive && (shell = getenv ("SYSTEM_SHELL")) != 0) {
	shelltype = getenv ("SYSTEM_SHELLTYPE");
	shellopt = getenv ("SYSTEM_SHELL_OPT");
    }
    else {
	shell = getenv ("SHELL");
	shellopt = getenv ("SHELL_OPT");
	shelltype = getenv ("SHELLTYPE");
    }

    /* VF^Cv 0 Ȃ... */
    if (shelltype == 0) {

	/* VFݒ肳Ă... */
	if (shell) {

	    /* ݒl̒𒲂ׂ */
	    len = strlen (shell);

	    /* ݒl */
	    if ((len >= 7 && stricmp (shell + len - 7, "command") == 0) ||
		(len >= 9 && stricmp (shell + len - 9, "command.x") == 0))
		shtype = SHTYPE_COMMAND;

	    /* Ȃ UNIX ^Cv... */
	    else
		shtype = SHTYPE_UNIX;

	}

	/* Ȃ COMMAND ^Cv... */
	else
	    shtype = SHTYPE_COMMAND;

    }

    /* VF^CvK肳Ă... */
    else {

	/* ݒ肩画f */
	if (stricmp (shelltype, "COMMAND") == 0)
	    shtype = SHTYPE_COMMAND;
	else if (stricmp (shelltype, "UNIX") == 0)
	    shtype = SHTYPE_UNIX;
	else
	    shtype = SHTYPE_UNKNOWN;

    }

    /* VF^CvƂ... */
    switch (shtype) {

    case SHTYPE_COMMAND:

	/*
	** COMMAND.X  HUPAIR GR[hꂽR}hC󂯎
	** Ȃ̂ŁAHUPAIR Ȃōs
	*/

	{
	    int std_fds[STDFDS];
	    char cmdshell[256];
	    char cwd[PATH_MAX + 1];
	    struct _comline comline;
	    struct _ontime t;

	    /* I */
	    if (shell == 0)
		shell = "COMMAND";
	    if (shellopt == 0)
		shellopt = "/C";

	    /* R}hC̒`FbN */
	    if ((strlen (shell) +
		 (interactive ? 0 : 2 /* space (0x20) x 2 */  +
		  strlen (shellopt) + strlen (cmdline))) > 255) {
		errno = E2BIG;
		return SYSTEM_ERR;
	    }

	    /* \ */
	    strcpy (cmdshell, shell);
	    if (!interactive) {
		strcat (cmdshell, " ");
		strcat (cmdshell, shellopt);
		strcat (cmdshell, " ");
		strcat (cmdshell, cmdline);
	    }

	    /* PATHCHK s */
	    if ((rc = _dos_pathchk (cmdshell, &comline, 0)) < 0) {
		errno = _errcnv (rc);
		return SYSTEM_ERR;
	    }

	    /* t@Cnhۑ */
	    for (i = 0; i < STDFDS; i++) {
		if ((std_fds[i] = dup (i)) < 0) {
		    for (i--; i >= 0; i--)
			close (std_fds[i]);
		    return SYSTEM_ERR;
		}
	    }

	    /* JgfBNgۑ */
	    if (_keep_cwd_on_exec)
		getcwd (cwd, sizeof cwd);

	    /* 荞݂~ */
	    _sigend ();

	    /* sO̎ԂL^ */
	    t = _iocs_ontime2 ();

	    /* s */
	    rc = _dos_loadexec (cmdshell, &comline, 0);

	    /* qvZX̎gpԂZ */
	    _chclock = _diffontime (_chclock, _diffontime (t, _iocs_ontime2 ()));

	    /* 荞݂ĊJ */
	    _siginit ();

	    /* JgfBNg߂ */
	    if (_keep_cwd_on_exec)
		chdir (cwd);

	    /* t@Cnh߂ */
	    for (i = 0; i < STDFDS; i++) {
		dup2 (std_fds[i], i);
		close (std_fds[i]);
	    }

	    /* G[`FbN */
	    if (rc < 0) {
		errno = _errcnv (rc);
		return SYSTEM_ERR;
	    }

	}

	break;

    case SHTYPE_UNIX:
    case SHTYPE_UNKNOWN:
    default:

	/* ΘbIȂ... */
	if (interactive)
	    rc = spawnlp (P_WAIT, shell, shell, (char *)NULL);

	/* Ȃ... */
	else {

	    /* IvV */
	    if (shellopt == 0)
		shellopt = "-c";

	    /* s */
	    rc = spawnlp (P_WAIT, shell, shell, shellopt, cmdline, (char *)NULL);

	}

	/* s? */
	if (rc == -1)
	    return SYSTEM_ERR;

	break;

    }

    /* ߂lԂ */
    return rc;
}
