/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: d_readlink.c,v 1.8 1994/11/26 15:03:02 mura Exp $
 */

/* System headers */
#include <stdio.h>
#include <string.h>
#include <sys/dos.h>
#include <sys/xglob.h>
#include <sys/xdoseml.h>

/* Typedefs */
typedef int (*func_1) (char *, const char *);
typedef int (*func_2) (const char *, unsigned short);

/* File scope functions */
static int realpathcopy (char *dst, const char *src)
{
#ifndef LNDRV_C_FUNCTION
    int ssp;
#endif
    int ret;
    int *lnenv;
    func_1 xpathcpy;

    /* lndrv 풓ĂȂ΃G[ */
    if ((lnenv = _dos_importlnenv ()) == 0)
	return -1;

#ifdef LNDRV_C_FUNCTION
    /* C_REALPATHCPY ̃xN^[AhX擾 */
    xpathcpy = (func_1) lnenv[C_REALPATHCPY];
    ret = xpathcpy (dst, src);
#else
    /* REALPATHCOPY ̃xN^[AhX擾 */
    xpathcpy = (func_1) lnenv[REALPATHCPY_VECTOR];

    /* X[p[oCU[[h lndrv 𒼐ڌĂяo */
    ssp = _dos_super (0);

    /* Ăяo */
    ret = CALL_XPATHCPY (xpathcpy, dst, src);

    /* ߂ */
    if (ssp > 0)
	_dos_super (ssp);
#endif

     /* ߂lԂ */
     return ret;
}

/* File scope functions */
static int orignal_dos_open (const char *file, unsigned short mode)
{
#ifndef LNDRV_C_FUNCTION
    int ssp;
#endif
    int ret;
    int *lnenv;
    func_2 xopen;

    /* lndrv 풓ĂȂ΃G[ */
    if ((lnenv = _dos_importlnenv ()) == 0)
	return -1;

#ifdef LNDRV_C_FUNCTION
    /* C_OLD_OPEN ̃xN^[AhX擾 */
    xopen = (func_2) lnenv[C_OLD_OPEN];
    ret = xopen (file, mode);
#else
    /* OLD_OPEN ̃xN^[AhX擾 */
    xopen = (func_2) lnenv[OLD_OPEN_VECTOR];

    /* X[p[oCU[[h lndrv 𒼐ڌĂяo */
    ssp = _dos_super (0);

    /* Ăяo */
    ret = CALL_XOPEN (xopen, file, mode);

    /* ߂ */
    if (ssp > 0)
	_dos_super (ssp);
#endif

    /* ߂lԂ */
    return ret;
}

/* Functions */
int _dos_readlink (const char *linkpath, char *namebuf, int buflen)
{
    char buf[PATH_MAX + 1];
    int len, fd, mode;

    /* linkpath ̃t@C𒲂ׂ */
    mode = _dos_chmod (linkpath, -1);

    /* t@CȂ */
    if (mode < 0)
	return _DOSE_NOENT;

    /* symbolic link łȂ sȃp[^ */
    if (!_DOS_ISLNK (mode))
	return _DOSE_ILGPARM;

    /* ۂ̃pX擾 G[Ȃ疳ȃt@NVR[ */
    if (realpathcopy (buf, linkpath) < 0)
	return _DOSE_ILGFNC;

    /* symbloic link t@CI[v */
    if ((fd = orignal_dos_open (buf, 0)) < 0)
	return fd;

    /* gǂݏo */
    len = _dos_read (fd, buf, PATH_MAX);

    /* t@CN[Y */
    _dos_close (fd);

    /* G[Aobt@zobt@I[o[ */
    if (len <= 0 || len > buflen)
	return _DOSE_BUFOVER;

    /* EOS ł */
    buf[len] = '\0';

    /* obt@Rs[ */
    strncpy (namebuf, _toslash (buf), buflen);

    /* Ԃ */
    return len;
}
