/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: _stat.c,v 1.15 1994/11/26 15:06:24 mura Exp $
 */

/* System headers */
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/dos.h>
#include <sys/xdoseml.h>
#include <sys/xglob.h>
#include <sys/xstat.h>
#include <sys/xstdlib.h>
#include <sys/xtime.h>
#include <sys/xunistd.h>

/* Macros */
#define SEARCH_ATTRIB    (0xff)
#define CHAIN_END_MARK   (0)
#define ROOTDIRECTORY(p) (p[0] != '\0' && p[1] == ':' && p[2] == '/' && p[3] == '\0')
#define PASSCURDRV(x)    ((x) + 1)

/* File scope functions */
static int _fakeroot (const char *, struct stat *, int);

/* Typedefs */
typedef int (*func) (const char *, unsigned short *, int);

/* File scope functions */
static int xlfatchk2 (const char *file, unsigned short *buf, int len)
{
#ifndef LNDRV_C_FUNCTION
    int ssp;
#endif
    int *lnenv;
    register int result;
    func lfatchk;

    /* lndrv ̊֐e[u߂ */
    if ((lnenv = _dos_importlnenv ()) == 0)
	return _dos_fatchk2 (file, buf, len);

    /* FATCHK2 s */
#ifdef LNDRV_C_FUNCTION
    lfatchk = (func) lnenv[C_LFATCHK];
    /* buf ̍ŏʃrbg𗧂 lndrv  lfatchk2 ĂяoB */
    buf = (unsigned short *) ((int) buf | (1 << 31));
    result = lfatchk (file, buf, len);
#else
    lfatchk = (func) lnenv[LFATCHK_VECTOR];
    ssp = _dos_super (0);
    result = CALL_XLFATCHK2 (lfatchk, file, buf, len);
    if (ssp > 0)
	_dos_super (ssp);
#endif

    /* ʂԂ */
    return result;
}

/* File scope functions */
static int _executable (const char *name)
{
    static const char* const exec_suffix[] = {
	"x", "r", 0
    };
    int i;
    char *top, *ext;

    /* t@C̈ʒu߂ */
    if ((top = strrchr (name, '/')) == 0)
	return 0;

    /* sIḧʒuT */
    if ((ext = strrchr (name, '.')) == 0)
	return 0;

    /* ̓t@C̒̃sIh */
    if (++ext <= top)
	return 0;

    /* gqXg猟 */
    for (i = 0; exec_suffix[i]; i++)
	if (stricmp (ext, exec_suffix[i]) == 0)
	    return 1;

    /* I */
    return 0;
}

/* File scope functions */
static int _setinodedirsize (const char *name, struct stat *st, struct _filbuf *fb)
{
    int length, i, err;
    fatchain_t fat;
    dpbsrc_t dpbsrc;
    struct _filbuf vdirfb;
    char xname[PATH_MAX + 1];

    /* fBNgȊOȂ i-node 쐬ďI */
    if (!S_ISDIR (st->st_mode)) {

	/* i-node vZ */
	if ((st->st_ino = _getinode (fb, WILD_FALSE)) < 0)
	    return -1;

	/* I */
	return 0;

    }

    /* Human  DPB ̃^lobt@ɓǂݍ */
    _getdpbsrc (fb->driveno, &dpbsrc);

    /* NĂ邩ǂ? */
    if (S_ISLNK (st->st_mode))
	err = _dos_fatchk2 (name, (unsigned short *) &fat, sizeof (fatchain_t) - 10);
    else
	err = xlfatchk2 (name, (unsigned short *) &fat, sizeof (fatchain_t) - 10);

    /* G[? */
    if (err < 0) {
	errno = _errcnv (err);
	return -1;
    }

    /* ChJ[h쐬... */
    strcat (strcpy (xname, name), "/*.*");

    /* mount point ̕␳ׁ̈AfBNg̃t@C */
    if (_dos_lfiles (&vdirfb, xname, SEARCH_ATTRIB) < 0)
	vdirfb.driveno = fb->driveno;

    /* mount point ̕␳Kv? */
    if (fb->driveno != vdirfb.driveno) {

	/* hCu␳ */
	st->st_dev = _phydrv2drvno (vdirfb.driveno);

	/* fBNgTCY߂ */
	st->st_size = dpbsrc.dircount * sizeof (direntry_t);

	/* i-inode 쐬 */
	st->st_ino = MAKEINODE (vdirfb.driveno, dpbsrc.dirsec);

    }
    else {

	/* fBNgTCY̌˂߂ */
	for (i = 0, length = 0; fat.chain[i].secno != CHAIN_END_MARK; i++)
	    length += fat.chain[i].seclen;

	/* fBNgTCY߂ */
	st->st_size = length * (dpbsrc.byte << dpbsrc.shift);

	/* i-inode 쐬 */
	st->st_ino = MAKEINODE (st->st_dev, fat.chain[0].secno);

    }

    /* I */
    return 0;
}

/* File scope functions */
static int _build_st (const char *name, struct stat *st,
		      struct _filbuf *fb, const char *node, int cdromflg)
{
    static const short days_in_month[12] = {
	0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
    };
    int days;
    time_t times;
    union {
	int dostime;
	tstamp_t tm;
    } ts;

    /* t@C[hZo */
    st->st_mode = _mode2unix (fb->atr);

    /* t@C[h̏C */
    if (_DOS_ISVOL (fb->atr))
	st->st_mode &= ~(S_IREAD | S_IWRITE);
    if (_executable (name))
	st->st_mode |= S_IEXEC;

    /* ^CX^v̎擾 */
    ts.dostime = (fb->date << 16) | fb->time;

    /* g[^̌vZ */
    days = ((ts.tm.year + 10) * 365
	    + (days_in_month[ts.tm.month - 1])
	    + (ts.tm.day)
	    + ((ts.tm.year >> 2) + 2)
	    - (((ts.tm.year & 3) == 0 && ts.tm.month < 3) ? 1 : 0));

    /* g[^b̌vZ (ŏ JST Ȃ̂ŕ␳) */
    times = (((days * 24 + ts.tm.hour) * 60
	      + ts.tm.minute) * 60
	     + (ts.tm.sec2 << 1) + _TZADJUST);

    /* Ԃ̐ݒ */
    st->st_atime = st->st_mtime = st->st_ctime = times;

    /* t@CTCYݒ(fBNgTCY̏ꍇ͌ŏ㏑) */
    st->st_size = fb->filelen;

    /* UID, GID, NLINK ݒ */
    st->st_uid = st->st_gid = 0;
    st->st_nlink = 1;

    /* ubNTCYݒ */
    st->st_blksize = 8192;

    /* _hCuݒ */
    st->st_rdev = _getdriveno (node);

    /* hCuݒ */
    st->st_dev = _phydrv2drvno (fb->driveno);

    /* CDROM ȊOȂ inode, dirsize ߂ďI */
    if(!cdromflg)
	return _setinodedirsize (name, st, fb);

    /* CDROM  S_IWRITE 폜 */
    st->st_mode &= ~S_IWRITE;

    /* CDROM  i-node Ƀ_~[ݒ */
    st->st_ino = 0;

    /* I */
    return 0;
}

/* File scope functions */
static int _fakecdev (struct stat *st, char *path)
{
    int ssp, fno;
    char xpath[PATH_MAX + 1];
    char *node;
    union _fcb fcb;

    /* V{bNNĂ邩H */
    node = _dos_readlink (path, xpath, sizeof xpath) > 0 ? xpath : path;

    /* ̓[gꍇ */
    if (ROOTDIRECTORY (node))
	return _fakeroot (node, st, 0);

    /* foCXI[v */
    if ((fno = _dos_open (node, O_RDONLY)) < 0) {
	errno = ENOENT;
	return -1;
    }

    /* FCB [JɃRs[Ă */
    ssp = _dos_super (0);
    memcpy (&fcb, _dos_getfcb (fno), sizeof (union _fcb));
    if (ssp > 0) _dos_super (ssp);

    /* foCXhCõGgAhX st_dev, st_rdev ɐݒ肷 */
    st->st_rdev = st->st_dev = (int) fcb.chr.deventry;

    /* foCXN[Y */
    _dos_close (fno);

    /* _~[ st \ */
    st->st_ino = st->st_gid = st->st_size = 0;
    st->st_atime = st->st_mtime = st->st_ctime = 0;
    st->st_nlink = 1;
    st->st_mode = S_IFCHR | S_IWRITE | S_IREAD;

    /* ubNTCYݒ */
    st->st_blksize = 4096;

    /* LN^^foCXłȂ΃G[ */
    if (!fcb.blk.mode) {
	errno = ENOENT;
	return -1;
    }

    /* I */
    return 0;
}

/* Functions */
int _statsub (int lstat, const char *name, struct stat *st)
{
    struct _filbuf fb;
    int status;
    int cdromflg;
    char node[PATH_MAX + 1];
    char expname[PATH_MAX + 1];

    /* t@C node ɃRs[ */
    strcpy (node, name);

    /* tpXɓWJAfBNgGgׂ */
    if (_fullpath (expname, name, sizeof expname) == NULL)
	return -1;

    /* CD-ROM hCuǂ */
    cdromflg = _iscdromdrive (PASSCURDRV (expname[0] - 'A'));

    /* [gfBNg */
    if (ROOTDIRECTORY (expname))
	return _fakeroot (expname, st, cdromflg);

    /* Ƃ肠ALFILES Ă݂ */
    if (lstat)
	status = _dos_lfiles (&fb, expname, SEARCH_ATTRIB);

    /* Ƃ肠 FILES  */
    else
	status = _dos_files (&fb, expname, SEARCH_ATTRIB);

    /* t@CȂfoCXł */
    if (status < 0)
	return _fakecdev (st, expname);

    /* t@C܂̓fBNg */
    else
	return _build_st (expname, st, &fb, node, cdromflg);
}

/* File scope functions */
static int _fakeroot (const char *name, struct stat *st, int cdromflg)
{
    int err;
    struct _dpbptr dpb;

    /* CDROM ̏ꍇ */
    if(cdromflg) {

	/* CDROM Ȃ_~[ */
	st->st_dev = st->st_rdev = _getdriveno (name);
	st->st_ino = 0;
	st->st_size = 0;
	st->st_mode = S_IFDIR | S_IREAD | S_IEXEC;
	st->st_uid = st->st_gid = 0;
	st->st_atime = st->st_mtime = st->st_ctime = 0;
	st->st_nlink = 1;

    }

    /* ȊO */
    else {

        /* DPB obt@ɓǂݍ */
        if ((err = _dos_getdpb (PASSCURDRV (_getdriveno (name)), &dpb)) < 0) {
	    errno = _errcnv (err);
	    return -1;
	}

	/* st \ */
	st->st_dev = st->st_rdev = dpb.drive;

	st->st_ino = MAKEINODE (dpb.drive, dpb.dirsec);
	st->st_size = dpb.dircount * sizeof (direntry_t);
	st->st_mode = S_IFDIR | S_IWRITE | S_IREAD | S_IEXEC;
	st->st_uid = st->st_gid = 0;
	st->st_atime = st->st_mtime = st->st_ctime = 0;
	st->st_nlink = 1;

    }

    /* I */
    return 0;
}
