/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: chmod.c,v 1.4 1994/05/02 17:21:23 mura Exp $
 */

/* System headers */
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <sys/stat.h>
#include <sys/xglob.h>
#include <sys/xunistd.h>

/* Macros */
#define OLDMASK (S_IEXEC | S_IREAD | S_IWRITE | S_IRDONLY | S_IEXBIT)
#define NEWMASK (S_IEXEC | S_IREAD | S_IWRITE)

/* Functions */
int chmod (const char *name, mode_t mode)
{
    dosmode_t oldattr, newattr;
    char xname[PATH_MAX + 1];

    /* Ō / 菜 */
    _dellastsep (strcpy (xname, name));

    /* ݂̃t@C[h𒲂ׂ */
    if ((oldattr = _dos_chmod (xname, -1)) < 0) {
	errno = _errcnv ((int) oldattr);
	return -1;
    }

    /* V[hvZ */
    {
	mode_t oldmode, newmode;

	oldmode = _mode2unix (oldattr);
	newmode = (oldmode & ~OLDMASK) | (mode & NEWMASK);
	newattr = _mode2dos (newmode);
    }

    /* V[hɕύX */
    if (oldattr != newattr) {
	int rc;

	if ((rc = _dos_chmod (xname, newattr)) < 0) {
	    errno = _errcnv (rc);
	    return -1;
	}
    }

    /* I */
    return 0;
}
