/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: fstat.c,v 1.5 1994/11/26 15:06:37 mura Exp $
 */

/* System headers */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/dos.h>
#include <sys/stat.h>
#include <sys/xglob.h>
#include <sys/xstat.h>
#include <sys/xunistd.h>
#include <sys/xtime.h>

/* Macros */
#define PASSCURDRV(x) ((x) + 1)

/* File scope functions */
static int _executable (const char *ext)
{
    static const char* const exec_suffix[] = {
	"x", "r", 0
    };
    int i;
    char xext[4];
    char *p;

    /* ی */
    xext[3] = '\0';

    /* gqo */
    if ((p = strchr (strncpy (xext, ext, 3), ' ')) != 0)
	*p = '\0';

    /* gqXg猟 */
    for (i = 0; exec_suffix[i]; i++)
	if (stricmp (xext, exec_suffix[i]) == 0)
	    return 1;

    /* Ȃ */
    return 0;
}

/* File scope functions */
static int _fstatsub (int fd, struct stat *st)
{
    static const short days_in_month[12] = {
	0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
    };
    int days;
    int topsec;
    time_t times;
    dpbsrc_t dpbsrc;
    union _fcb *fcb;
    union {
	int dostime;
	tstamp_t tm;
    } ts;

    /* FCB ߂ */
    fcb = _dos_getfcb (fd);

    /* foCX(LN^^)H */
    if (fcb->blk.mode) {

	/* _~[ st \ */
	st->st_ino = st->st_gid = st->st_size = 0;
	st->st_atime = st->st_mtime = st->st_ctime = 0;
	st->st_nlink = 1;
	st->st_rdev = st->st_dev = (int) fcb->chr.deventry;
	st->st_mode = S_IFCHR | S_IWRITE | S_IREAD;

	/* ubNTCYݒ */
	st->st_blksize = 4096;

	return 0;
    }

    /* t@C[hZo */
    st->st_mode = _mode2unix (fcb->blk.attr);

    /* t@C[h̏C */
    if (_executable (fcb->blk.ext))
	st->st_mode |= S_IEXEC;

    /* ^CX^v̎擾 */
    ts.dostime = (fcb->blk.date << 16) | fcb->blk.time;

    /* g[^̌vZ */
    days = ((ts.tm.year + 10) * 365
	    + (days_in_month[ts.tm.month - 1])
	    + (ts.tm.day)
	    + ((ts.tm.year >> 2) + 2)
	    - (((ts.tm.year & 3) == 0 && ts.tm.month < 3) ? 1 : 0));

    /* g[^b̌vZ (ŏ JST Ȃ̂ŕ␳) */
    times = (((days * 24 + ts.tm.hour) * 60
	      + ts.tm.minute) * 60
	     + (ts.tm.sec2 << 1) + _TZADJUST);

    /* Ԃ̐ݒ */
    st->st_atime = st->st_mtime = st->st_ctime = times;

    /* t@CTCYݒ */
    st->st_size = fcb->blk.size;

    /* UID, GID, NLINK ݒ */
    st->st_uid = st->st_gid = 0;
    st->st_nlink = 1;
    st->st_dev = st->st_rdev = _phydrv2drvno (fcb->blk.physdrv);

    /* ubNTCYݒ */
    st->st_blksize = 8192;

    /* CD-ROM hCuȂ... */
    if (!_iscdromdrive (PASSCURDRV (st->st_dev))) {

	/* Human  DPB ̃^lobt@ɓǂݍ */
	_getdpbsrc (fcb->blk.physdrv, &dpbsrc);

	/* FATԍZN^(NX^)ԍɕϊ */
	topsec = dpbsrc.datasec + ((int) (fcb->blk.fatno - 2) << dpbsrc.shift);

	/* INODE 쐬 */
	st->st_ino = MAKEINODE (st->st_dev, topsec);

    }

    /* CD-ROM hCu */
    else {

	/* CDROM  S_IWRITE 폜 */
	st->st_mode &= S_IWRITE;

	/* CDROM  i-node Ƀ_~[ݒ */
	st->st_ino = 0;

    }

    /* I */
    return 0;
}

/* Functions */
int fstat (int fd, struct stat *st)
{
    int ssp, result;

#ifdef EBADF_CHECKER
    /* t@Cnh`FbN */
    if (_usedfd (fd) < 0) {
	errno = EBADF;
	return -1;
    }
#endif

    /* X[p[oCU[[h */
    ssp = _dos_super (0);

    /* stat s */
    result = _fstatsub (fd, st);

    /* ɖ߂ */
    if (ssp > 0)
	_dos_super (ssp);

    /* ʂԂ */
    return result;
}
