/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: mktime.c,v 1.1.1.1 1993/04/18 16:38:30 mura Exp $
 */

/* System headers */
#include <time.h>
#include <limits.h>
#include <string.h>
#include <errno.h>
#include <sys/xtime.h>

/* Functions */
time_t mktime (struct tm *timeptr)
{
    int y, m, d, s;
    long savtz;
    double ts;
    time_t retval;

    /* n掞ԏ */
    tzset ();

    /* N␳ */
    m = timeptr->tm_year * 12 + timeptr->tm_mon;
    y = m / 12 + 1900;
    m = m % 12;
    d = 365 * (y - 1970) + _getleaps (y)
	+ _getyday (m, y) + timeptr->tm_mday - 1;

    /* ̓̓̕bvZ */
    s = 3600 * timeptr->tm_hour
	+ 60 * timeptr->tm_min + timeptr->tm_sec;

    /* g[^bvZ */
    ts = (double) d * 86400.0 + (double) s;

    /* ԃ`FbN */
    if (ts < 0.0 || ts > (double) LONG_MAX) {
	errno = EINVAL;
	return -1;
    }

    /* time_t ɕϊ */
    retval = (time_t) ts;

    /* ꎞIɃ^C][ޔ */
    savtz = timezone;

    /* isdst Ȃ΃T}[^C肷 */
    if (timeptr->tm_isdst > 0) {
	retval += timezone_dst;
	timezone = timezone_dst;
    }

    /* isdst  0 Ȃ΃T}[^CłȂƉ肷 */
    else if (timeptr->tm_isdst == 0) {
	retval += timezone_std;
	timezone = timezone_std;
    }

    /* isdst Ȃ΃T}[^Cǂ݂͌̏Ԃɂ */
    else {
	retval += timezone;
	timezone = timezone;
    }

    /* ␳̒l܂n掞Ԃɖ߂ */
    *timeptr = _localtime (&retval);

    /* ޔĂ^C][߂ */
    timezone = savtz;

    /* b߂ */
    return retval;
}
