/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: tzset.c,v 1.1.1.1 1993/04/18 16:38:38 mura Exp $
 */

/* System headers */
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <sys/xtime.h>

/* External variables */
char *tzname[2];
int daylight;
long timezone;
long timezone_dst;
long timezone_std;

/* File scope functions */
static int daydigit_get (char *, int);
static int julian_get (int, int, int);
static int monthsday_get (int, int);
static int timezone_set (int);
static int weekofmonths (int, int);
static int zonetime_set (long *);
static time_t zonetime_sub (void);

/* File scope variables */
static unsigned char *envptr;

/*
** nԂɊւϐ TZ 擾AO[oϐɃZbg
*/
void tzset (void)
{
    int dstflag = 0;

#define ERROR_CHECK() if (*envptr == '\0') goto bailout

    /*
    ** ϐ TZ 荞݂܂
    */
    if ((envptr = getenv ("TZ")) == 0) {
	daylight = 0;
	tzname[0] = "JST";
	tzname[1] = "   ";
	timezone_std = timezone_dst = timezone = -9 * 60 * 60;
	return;
    }

    /*
    ** ܂^C][
    */
    daylight = 0;
    timezone = timezone_dst = timezone_std = 0;

    /*
    ** ŏ̃tB[hun掞ԑт̖́vo܂
    */
    ERROR_CHECK ();
    envptr += timezone_set (0);

    /*
    ** QtB[h́uWƒn̍ (PʁEb)vo܂
    */
    ERROR_CHECK ();
    envptr += zonetime_set (&timezone_std);
    timezone_dst = timezone_std;

    /*
    ** R̃tB[hun掞ԑт̉Ďԑт̖́vo܂
    */
    ERROR_CHECK ();
    envptr += timezone_set (1);

    /*
    ** StB[h́uWƉĎԂ̍ (PʁEb)vo܂
    */
    ERROR_CHECK ();
    envptr += zonetime_set (&timezone_dst);

    /*
    ** ĎԂ̊Jn
    */
    ERROR_CHECK ();
    if (*envptr == 'M')
	dstflag = monthsday_get (dstflag, 1);
    else
	dstflag = julian_get (dstflag, 1, *envptr == 'J');

    /* Zp[^`FbN */
    ERROR_CHECK ();
    if (*envptr == ',')
	envptr++;

    /*
    ** ĎԂ̏I
    */
    ERROR_CHECK ();
    if (dstflag == 1) {
	if (*envptr == 'M')
	    dstflag = monthsday_get (dstflag, 0);
	else
	    dstflag = julian_get (dstflag, 0, *envptr == 'J');
    }

  bailout:

    /* timezone ݒ */
    timezone = (dstflag) ? timezone_dst : timezone_std;

    /* ĎԐݒ */
    daylight = dstflag;
}

/*
** Julian t𓾂
** ĎԐݒtԃtB[h J tH[}bg̉
*/
static int julian_get (int dst, int mode, int julian)
{
    int wk, yday;
    struct tm tm;
    time_t timebuf1, timebuf2;
    char tims[3] = {0, 0, 0};

    /* J XLbv */
    if (julian)
	envptr++;

    /* Julian t擾 */
    yday = 0;
    while (isdigit (*envptr)) {
	yday = yday * 10 + (*envptr - '0');
	envptr++;
    }

    /* ͈̓`FbN */
    if (yday < julian || yday > 365)
	return 0;

    /* |C^ */
    if (*envptr)
	envptr++;

    /* Ԃ擾 */
    wk = daydigit_get (tims, 1);

    /* s? */
    if (wk != 2) {
	tims[0] = 2;
	tims[1] = 0;
	tims[2] = 0;
    }

    /* ݂̒n掞Ԃ߂... (ŒIł...悭Ȃ) */
    timezone = _TZADJUST;
    timebuf1 = time (0) - timezone;
    tm = _gmtime (&timebuf1);

    /* February 29 ̓Ꮘ */
    if (julian && _isleap (2, tm.tm_year + 1970))
	tm.tm_yday--;

    /* Julian t̎ԕb... */
    timebuf1 = tims[0] * 3600 + tims[1] * 60 + tims[2];

    /* n掞Ԃ̎ԕb... */
    timebuf2 = tm.tm_hour * 3600 + tm.tm_min * 60 + tm.tm_sec;

    /* Ďԃ`FbN */
    if ((tm.tm_yday > yday) ||
	(tm.tm_yday == yday && timebuf2 >= timebuf1))
	return mode;
    else
	return dst;
}

/*
** ĎԐݒtԃtB[h M tH[}bg̉
*/
static int monthsday_get (int dst, int mode)
{
    int wk, dstflag;
    char days[3] = {0, 0, 0};
    char tims[3] = {0, 0, 0};

    /* M XLbv */
    envptr++;

    /* t擾 */
    wk = daydigit_get (days, 0);

    /* dstflag ̏l */
    dstflag = dst;

    /* tB[h 2? */
    if (wk == 2) {

	/* XLbv... */
	envptr++;

	/* Ԃ擾 */
	wk = daydigit_get (tims, 1);

	/* G[? */
	if (wk != 2) {
	    tims[0] = 2;
	    tims[1] = 0;
	    tims[2] = 0;
	}

	/* G[łȂ... */
	if (days[0] >= 1 && days[0] <= 12 &&
	    days[1] >= 1 && days[1] <= 5 &&
	    days[2] >= 0 && days[2] <= 6) {

	    time_t timebuf1, timebuf2;
	    struct tm tm;

	    /* ݂̒n掞Ԃ𓾂 */
	    timezone = -32400;
	    timebuf1 = time (0) - timezone;
	    tm = _gmtime (&timebuf1);

	    /* M t̎ԕb ... */
	    timebuf1 = tims[0] * 3600 + tims[1] * 60 + tims[2];

	    /* n掞Ԃ̎ԕb ... */
	    timebuf2 = tm.tm_hour * 3600 + tm.tm_min * 60 + tm.tm_sec;

	    /* w茎zĂ? */
	    if (tm.tm_mon > (days[0] - 1))
		dstflag = mode;

	    /* ? */
	    else if (tm.tm_mon == (days[0] - 1)) {

		/* łɎwTzĂ? */
		if (weekofmonths (tm.tm_mday, tm.tm_wday) > days[1])
		    dstflag = mode;

		/* T? */
		else if (weekofmonths (tm.tm_mday, tm.tm_wday) == days[1]) {

		    /* łɎwjzĂ? */
		    if (tm.tm_wday > days[2])
			dstflag = mode;

		    /* w莞ԂzĂ? */
		    else if ((tm.tm_wday == days[2]) && (timebuf2 >= timebuf1))
			dstflag = mode;

		}

	    }
	}
    }

    /* ĎԃtOԂ */
    return dstflag;
}

/*
** ̌̑扽TԂ ( 1 TJEg)
*/
static int weekofmonths (int md, int wd)
{
    int temp;

    /* TvZ */
    temp = md - wd - 1;
    return (md < wd) ? 1 : (temp / 7) + ((temp % 7) ? 2 : 1);
}

/*
** timezone ̐ݒ ::
**	envptr T[`Atzname[mode] ɃRs[Bݒ肪
**	ꍇ̓ftHgZbg
*/
static int timezone_set (int mode)
{
    int cnt, len;

    /* ][̒𒲂ׂ */
    cnt = 0;
    while (!isdigit (envptr[cnt]) && envptr[cnt] != '-' &&
	   envptr[cnt] != '+' && envptr[cnt] != ',' && envptr[cnt] != ':')
	cnt++;

    /* 0 ȂΏI */
    if (cnt == 0)
	return 0;

    /* 31 蒷Ă܂ */
    if ((len = cnt) > 31)
	len = 31;

    /* ̈m */
    if (tzname[mode] == 0)
	tzname[mode] = malloc (len + 1);

    /* obt@ɃRs[ */
    if (tzname[mode]) {
	strncpy (tzname[mode], envptr, len);
	tzname[mode][len] = '\0';
    }

    /* Ō "," ":" XLbv */
    if (envptr[cnt] == ',' || envptr[cnt] == ':')
	cnt++;

    /* i񂾃oCgԂ */
    return cnt;
}

/*
** n掞Ԃ UTC ̍߂ ::
**	ʂ́Azonetime Ɋi[B
*/
static int zonetime_set (long *zonetime)
{
    int cnt;
    long zone;
    time_t hour, min;
    time_t sec, sign;

    /* JE^/ */
    cnt = 0;
    sign = 0;

    /* L */
    if (*envptr == '+')
	envptr++;
    else if (*envptr == '-') {
	sign = 1;
	envptr++;
    }

    /* ::b擾 */
    hour = zonetime_sub ();
    min = (isdigit (*envptr)) ? zonetime_sub (): 0;
    sec = (isdigit (*envptr)) ? zonetime_sub (): 0;

    /* vZ */
    zone  = (hour < 24 && hour > -24) ? hour * 60 * 60 : -9 * 60 * 60;
    zone += (min < 60 && min > 0) ? min * 60 : 0;
    zone += (sec < 60 && sec > 0) ? sec : 0;
    *zonetime = sign ? -zone : zone;

    /* Ō "," ":" XLbv */
    if (*envptr == ',' || *envptr == ':')
	cnt++;

    /* i񂾃oCgԂ (sub ŏ͏) */
    return cnt;
}

/*
** ԃtB[ĥϊ ::
**	envptr T[`AlɕϊĕԂB
*/
static time_t zonetime_sub (void)
{
    int cnt;
    time_t timeret;
    char temp[5];

    /* l̐vZ */
    cnt = 0;
    while (isdigit (envptr[cnt]))
	cnt++;

    /*  0 ȂI */
    if (cnt == 0)
	return 0;

    /* lRs[ */
    strncpy (temp, envptr, cnt);
    temp[cnt] = '\0';

    /* ɕϊ */
    timeret = atoi (temp);

    /* |C^i߂ */
    envptr += cnt;

    /* ':' XLbv */
    if (*envptr == ':')
	envptr++;

    /* ϊʂԂ */
    return timeret;
}

/*
** ttB[h𓾂 ::
**	envptr T[`Aarray zɃZbgBtAԂ
**	ʂ mode == 0 œtAmode != 0 Ŏ
*/
static int daydigit_get (char *array, int mode)
{
    int wk, sep;

    /* Zp[^𓾂 */
    wk = 0;
    sep = (mode == 0) ? '.' : ':';

    /* envptr 𒲂ׂ */
    while (*envptr) {

	/* l擾 */
	if (isdigit (*envptr))
	    array[wk] = array[wk] * 10 + (*envptr - '0');
	else if (*envptr == sep)
	    wk++;
	else
	    break;

	/* ̕... */
	envptr++;

    }

    /* tB[hԂ */
    return wk;
}
