/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * based on the contribution from hupairlib by K.Abe (Abechan).
 * --------------------------------------------------------------------
 * $Id: _spawn.c,v 1.5 1994/11/27 13:10:08 mura Exp $
 */

/* System headers */
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/dos.h>
#include <sys/iocs.h>
#include <sys/xglob.h>
#include <sys/xtime.h>
#include <sys/xstart.h>
#include <sys/xsignal.h>
#include <sys/xunistd.h>

/* Macros */
#define INITIAL_BUFSIZ 1024
#define STDFDS 5

/* External variables */
int _hugearg = _HUGEARG_DEFAULT;
int _keep_cwd_on_exec = 1;
char *_hugearg_indirect;

/* File scope variables */
static long nullenv[2] = {5, 0};

/* break check save */
static int obreak;

/* Structures and unions */
enum exec_mode {
    EXEC_LOADEXEC = 0,
    EXEC_LOAD,
    EXEC_PATHCHK,
    EXEC_LOADONLY,
    EXEC_EXECONLY,
    EXEC_BINDNO,
};

/* File scope functios */
static int indirect_create (char *arg, char *buffer)
{
    int fd;
    char *dir;
    char *iname;

    /* ev[g쐬 */
    if ((dir = getenv ("temp")) == 0) {
	strcpy (buffer, "#exeXXXXXX");
    }
    else {
	_addlastsep (strcpy (buffer, dir));
	_tobslash (strcat (buffer, "#exeXXXXXX"));
    }

    /* C_CNgt@C쐬 */
    if ((int) (iname = mktemp (buffer)) < 0)
	return -1;

    /* C_CNgt@C쐬 */
    if ((fd = open (iname, O_WRONLY | O_TRUNC | O_CREAT, 0666)) < 0)
	return -1;

    /* e */
    if (write (fd, arg, strlen (arg)) < 0) {
	close (fd);
	unlink (iname);
	return -1;
    }

    /* t@CN[YďI */
    close (fd);

    /*  */
    return 0;
}

/* File scope functions */
static int indirect_exec (char *base, char *cmdline,
			  struct _dregs *regptr, const char *argv0)
{
    int rc;
    char iname[PATH_MAX + 1];
    struct _ontime t;

    /* C_CNgt@C쐬 */
    if (indirect_create (cmdline + 1, iname) < 0) {
	_noexec_return (regptr);
	errno = EINVAL;
	return -1;
    }

    /* ۂɃR}hCɓn쐬 */
    strcat (strcpy (cmdline + 1, base), iname);
    cmdline[0] = strlen (cmdline + 1) & 255;

    /* { break  */
    _dos_breakck (obreak);

    /* sO̎ԂL^ */
    t = _iocs_ontime2 ();

    /* vOs */
    rc = _exec (EXEC_EXECONLY, regptr->a4, (long) cmdline, regptr->a3, regptr);

    /* qvZX̎gpԂZ */
    _chclock = _diffontime (_chclock, _diffontime (t, _iocs_ontime2 ()));

    /* q break ς̈ */
    obreak = _dos_breakck (-1);
    _dos_breakck (2);

    /* C_CNgt@C폜 */
    unlink (iname);

    /* G[`FbN */
    if (rc < 0) {
	errno = _errcnv (rc);
	return -1;
    }

    /*  */
    return rc;
}

/* File scope functios */
static char *copy_environ (char *const envp[])
{
    int envsize;
    char *dos_envbuf, *dos_envp, *q;
    char *const *ptr;

    /* envp kȂ environ p */
    if (envp == 0)
	envp = (char **) environ;

    /* GÃg[^TCY߂ */
    for (ptr = envp, envsize = 4 + 1; *ptr; ptr++)
	envsize += strlen (*ptr) + 1;

    /* o_ɒ */
    envsize = (envsize + 1) & ~1;

    /* Rs[̗̈mۂ */
    if ((dos_envbuf = malloc (envsize)) == 0)
	return 0;

    /* 擪ɃoCgݒ */
    *(unsigned long *) dos_envbuf = envsize;

    /* VGAɂׂăRs[ */
    for (ptr = envp, dos_envp = dos_envbuf + 4; (q = *ptr++);)
	while ((*dos_envp++ = *q++) != 0);

    /* ŌkŎ~߂ */
    *dos_envp++ = '\0';

    /* V̈Ԃ */
    return dos_envbuf;
}

/* Functions */
int _spawn (int mode, const char *pathname,
	    char *const argv[], char *const envp[], int search_path)
{
    char *child_envp;						/* qvZX̊ */
    char *cmdline = 0;						/* R}hC */
    char fpath[PATH_MAX + 1];					/* spX */
    char cwd[PATH_MAX + 1];					/* JgfBNg */
    const char *argv0;						/* ␳ argv[0] */
    int cmdlen;							/* R}hC̒ */
    int rc = 0;							/* ^[R[h */
    int exec_success = 0;					/* */
    int std_fds[STDFDS];					/* t@Cnhۑp */
    struct _dregs regs;						/* ۑWX^ */

    enum {
	JUST_BEGIN, AFTER_ENVSET, AFTER_ENCODE, AFTER_SIGEND
    } state = JUST_BEGIN;					/* sXe[gtO */

#define RETURN(x) { rc = (x); goto bailout; }

    /*
    ** (1) G[`FbNyя
    */

    {
	/* `FbN */
	if (mode != P_WAIT && mode != P_OVERLAY) {
	    errno = EINVAL;
	    return -1;
	}

	/* _hugearg `FbN */
	if (_hugearg != _HUGEARG_DEFAULT &&
	    _hugearg != _HUGEARG_FORCE &&
	    _hugearg != _HUGEARG_NOEXEC &&
	    _hugearg != _HUGEARG_INDIRECT) {
	    errno = EINVAL;
	    return -1;
	}

	/* qvZXpGA쐬 */
	if ((child_envp = copy_environ (envp)) == 0)
	    RETURN (-1);

	/* Xe[gtOݒ */
	state = AFTER_ENVSET;
    }

    /*
    ** (2) st@C.....
    */

    {
	char tmparg[256];					/* e| */

	/* pX`FbNs */
	strcpy (fpath, pathname);
	if ((rc = _exec (EXEC_PATHCHK, (long) fpath, (long) tmparg,
			 (long) (search_path ? 0 : nullenv), &regs)) < 0) {
	    errno = _errcnv (rc);
	    RETURN (-1);
	}
    }

    /*
    ** (3) R}hC HUPAIR GR[h ARGV 쐬.....
    */

    {
	int bufsiz = INITIAL_BUFSIZ;				/* obt@TCY */

	/* argv[0] ̒s */
	argv0 = argv[0] ? : pathname;

	/* R}hCp̃obt@mۂ */
	if ((cmdline = malloc (bufsiz)) == 0)
	    RETURN (-1);

	/* argv S쐬܂Ń[v */
	do {

	    /* R}hC HUPAIR GR[h */
	    if ((cmdlen = _enhupair (argv0, argv + 1,
				     cmdline + 8, bufsiz - 8)) < 0) {

		/* obt@Ȃ΂ǂǂǉ */
		bufsiz += 1024;

		/* obt@Ċ蓖Ăs */
		if ((cmdline = realloc (cmdline, bufsiz)) == 0)
		    RETURN (-1);

	    }

	} while (cmdlen < 0);

    }

    /*
    ** (4) `FbNpɃvO[h.....
    */

    {
	int i;							/* JE^ */

	/* Xe[gtOݒ */
	state = AFTER_ENCODE;

	/* t@Cnhۑ */
	for (i = 0; i < STDFDS; i++)
	    if ((std_fds[i] = dup (i)) < 0)
		RETURN (-1);

	/* 荞ݒ~ */
	_sigend ();

	/* ԑJ */
	state = AFTER_SIGEND;

	/* break  kill ɂ */
	obreak = _dos_breakck (-1);
	_dos_breakck (2);

	/* vO[h */
	rc = _exec (EXEC_LOAD, (long) fpath,
		    (long) cmdline + 8, (long) child_envp, &regs);

	/* G[ */
	if (rc < 0) {
	    errno = _errcnv (rc);
	    RETURN (-1);
	}

	/* JgfBNgۑ */
	if (_keep_cwd_on_exec)
	    getcwd (cwd, sizeof cwd);

    }

    /*
    ** (5) vO HUPAIR ǂŃR}hC̓n߂.....
    */

    {
	char *arg;						/* cmdline + 8 */
	char *hugearg;						/* HUGEARG ̐ݒ */
	char *indstr;						/* indirect  */
	int ishupair;						/* HUPAIR tO */

	/* HUPAIR ǂׂ */
	ishupair = (strcmp ((char *) regs.a4 + 2, _hupair_mark) == 0);

	/* cmdline + 8 ߂ */
	arg = (char *) regs.a2;

	/* HUPAIR 邢 255 ȓȂ ... */
	if (ishupair || cmdlen <= 255) {
	    struct _ontime t;

	force:

	    /* { break  */
	    _dos_breakck (obreak);

	    /* sO̎ԂL^ */
	    t = _iocs_ontime2 ();

	    /* vOs */
	    rc = _exec (EXEC_EXECONLY, regs.a4, (long) arg, regs.a3, &regs);

	    /* qvZX̎gpԂZ */
	    _chclock = _diffontime (_chclock, _diffontime (t, _iocs_ontime2 ()));

	    /* q break ς̈ */
	    obreak = _dos_breakck (-1);
	    _dos_breakck (2);

	    /* G[ */
	    if (rc < 0) {
		errno = _errcnv (rc);
		RETURN (-1);
	    }

	}
	else {

	    /* ϐ HUGEARG  */
	    hugearg = getenv ("HUGEARG");

	    /* _hugearg ɂČ */
	    switch (_hugearg) {

	    case _HUGEARG_FORCE:

		/* IɎs */
		goto force;

	    case _HUGEARG_NOEXEC:

	    noexec:
		/* ssɂ */
		_noexec_return (&regs);
		errno = E2BIG;
		RETURN (-1);

	    case _HUGEARG_INDIRECT:

		/* C_CNg񂪎w肳ĂȂ΃G[ */
		if (_hugearg_indirect == 0) {
		    _noexec_return (&regs);
		    errno = EINVAL;
		    RETURN (-1);
		}

		/* C_CNgs */
		rc = indirect_exec (_hugearg_indirect, cmdline + 8, &regs, argv0);

		/* ̏ */
		break;

	    case _HUGEARG_DEFAULT:

		/* HUGEARG ݒ肳ĂȂΎss */
		if (hugearg == 0)
		    goto noexec;

		/* HUGEARG = force Ȃ΋s */
		if (strcmp (hugearg, "force") == 0)
		    goto force;

		/* HUGEARG = indirect łȂΌ̓G[ */
		if (strncmp (hugearg, "indirect", 8) != 0)
		    goto noexec;

		/* C_CNg̎o */
		indstr = hugearg[8] ? &hugearg[9] : "-+-+-";

		/* C_CNgs */
		rc = indirect_exec (indstr, cmdline + 8, &regs, argv0);

		/* ̏ */
		break;

	    default:

		/* YȂꍇ̓G[ */
		goto noexec;
	    }
	}

	/* ? */
	exec_success = 1;

	/* JgfBNg߂ */
	if (_keep_cwd_on_exec)
	    chdir (cwd);

    }

    /*
    ** (6) ㏈s
    */

    {
	int i;							/* JE^ */

    bailout:

	/* sXe[gɉă */
	switch (state) {

	case AFTER_SIGEND:

	    /* break tO߂ */
	    _dos_breakck (obreak);

	    /* 荞ݍĊJ */
	    _siginit ();

	case AFTER_ENCODE:

	    /*  */
	    free (cmdline);

	    /* t@Cnh߂ */
	    for (i = 0; i < STDFDS && std_fds[i] >= 0; i++) {
		dup2 (std_fds[i], i);
		close (std_fds[i]);
	    }

	case AFTER_ENVSET:

	    /*  */
	    free (child_envp);

	case JUST_BEGIN:

	    /* ̏ */
	    break;

	}
    }

    /* I[o[C̏ */
    if (exec_success && mode == P_OVERLAY)
	exit (rc);

    /* ^[R[h߂ */
    return rc;
}
