/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: read.c,v 1.7 1994/11/26 15:11:28 mura Exp $
 */

/* System headers */
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/dos.h>
#include <sys/xglob.h>
#include <sys/xunistd.h>

/* Functions */
static int raw_keyboard (int fileno, char *buffer, unsigned int bytes)
{
    int n, rc, md;
    char *p;

    /* [ho */
    md = _fddb[fileno].oflag;

    /* eLXg[h EOF Ȃ炱ɂďI */
    if ((md & O_TEXT) && (md & O_TXTEOF))
	return 0;

    /* ǂ߂邾ǂ */
    if ((rc = _dos_read (fileno, buffer, bytes)) < 0) {
	errno = _errcnv (rc);
	return -1;
    }

    /* oCi[hȂ牽Ȃ */
    if (md & O_BINARY)
	return rc;

    /* CR -> LF ϊ, ^Z -> EOF  */
    for (n = rc, p = buffer; n--; p++) {

	/* EOF(^Z) ɂł... */
	if (*p == 0x1a) {

	    /* TEXTEOF tOݒ */
	    _fddb[fileno].oflag |= O_TXTEOF;
	    break;

	}

	/* CR ɂł... */
	else if (*p == '\r')
	    *p = '\n';

    }

    /* oCg߂ */
    return p - buffer;
}

/* Functions */
static int cooked_keyboard (int fileno, char *buffer, unsigned int bytes)
{
    int n, rc, md;
    char ch, *rp, *wp;

    /* [ho */
    md = _fddb[fileno].oflag;

    /* eLXg[h EOF Ȃ炱ɂďI */
    if ((md & O_TEXT) && (md & O_TXTEOF))
	return 0;

    /* ǂ߂邾ǂ */
    if ((rc = _dos_read (fileno, buffer, bytes)) < 0) {
	errno = _errcnv (rc);
	return -1;
    }

    /* oCi[hȂ牽Ȃ */
    if (md & O_BINARY)
	return rc;

    /* ϊp̕ϐ */
    n = rc;
    wp = rp = buffer;

    /* CR/LF -> LF ϊ, ^Z -> EOF  */
    while (n--) {

	/* EOF(^Z) ɂł... */
	if ((ch = *rp++) == 0x1a) {

	    /* TEXTEOF tOݒ */
	    _fddb[fileno].oflag |= O_TXTEOF;
	    break;

	}

	/* CR ɂł... cooked K CR,LF Ƒ */
	if (ch == '\r') {

	    /* obt@ɂȂ LF ǂݏo */
	    if (n == 0) {
		if ((rc = _dos_read (fileno, &ch, 1)) < 0) {
		    errno = _errcnv (rc);
		    return -1;
		}
	    }
	    else {
		ch = *rp++;
		n--;
	    }
	}

	/* f[^] */
	*wp++ = ch;

    }

    /* oCg߂ */
    return wp - buffer;

}

/* Functions */
int read (int fileno, char *buffer, unsigned int bytes)
{
    int n, rc;
    int count;
    long position;
    char ch, got;
    char *rp, *wp;

#ifdef EBADF_CHECKER
    /* sȃt@Cnh`FbN */
    if (_usedfd (fileno) < 0) {
	errno = EBADF;
	return -1;
    }
#endif

    /* 0 oCgȂ牽sȂ */
    if (bytes <= 0)
	return 0;

    /* L[{[h̓ǂݍ... */
    if (isatty (fileno)) {

	/* cooked/raw ̊mF */
	if ((rc = _dos_ioctrlgt (fileno)) < 0) {
	    errno = _errcnv (rc);
	    return -1;
	}

	/* raw foCX */
	else if (rc & _DEV_RAW)
	    return raw_keyboard (fileno, buffer, bytes);

	/* cooked foCX */
	else
	    return cooked_keyboard (fileno, buffer, bytes);
    }

    /* oCi[h̏ */
    if (_fddb[fileno].oflag & O_BINARY) {

	/* oCi[hł͂ǂނ */
	if ((count = _dos_read (fileno, buffer, bytes)) < 0) {
	    errno = _errcnv (count);
	    return -1;
	}

    }
    else {

	/* EOF Ȃ炱ɂďI */
	if (_fddb[fileno].oflag & O_TXTEOF)
	    return 0;

	/* ǂ߂邾ǂ */
	if ((rc = _dos_read (fileno, buffer, bytes)) < 0) {
	    errno = _errcnv (rc);
	    return -1;
	}

	/* ϊp̕ϐ */
	n = rc;
	wp = rp = buffer;
	count = 0;

	/* CR/LF -> LF ϊ */
	while (n--) {

	    /* EOF(^Z) ɂł... */
	    if ((ch = *rp++) == 0x1a) {

		/* TEXTEOF tOݒ */
		_fddb[fileno].oflag |= O_TXTEOF;
		break;

	    }

	    /* CR ɂł... */
	    if (ch == '\r') {

		/* obt@ɗ]f[^邩? */
		if (n) {

		    /*  LF ł CR 𔲂 */
		    if (*rp == '\n') {
			ch = *rp++;
			n--;
		    }

		}

		/* ]肪Ȃ΂ 1 oCgǂݏoĒׂ */
		else {

		    /* ܂݈ʒuL */
		    position = _dos_seek (fileno, 0, 1);

		    /* 1 oCgǂ݂ */
		    if ((rc = _dos_read (fileno, &got, 1)) < 0) {
			errno = _errcnv (rc);
			return -1;
		    }

		    /*  LF Ȃ CR 𔲂 */
		    if (got == '\n')
			ch = got;

		    /* LF Ȃ΃t@C|C^߂ */
		    else
			_dos_seek (fileno, position, 0);

		}

	    }

	    /* f[^] */
	    *wp++ = ch;

	}

	/* oCg߂ */
	count = wp - buffer;

    }

    /* ߂ */
    return count;
}
